@@ -270,7 +270,7 @@ ThreadStaticInit (Value thread, InstPtr *next)
     EXIT ();
 }
 
-static INLINE void
+static inline void
 ThreadAssign (Value ref, Value v, Bool initialize)
 {
     ENTER ();
@@ -1244,7 +1244,7 @@ ThreadsRun (Value thread, Value lex)
 						value);
 			break;
 		    }
-		    value = RefValue (value);
+		    value = RefValueGet(value);
 		    /* fall through ... */
 		case OpDot:
 		case OpDotRef:
@@ -1290,12 +1290,16 @@ ThreadsRun (Value thread, Value lex)
 		    break;
 		case OpPreOp:
 		    v = Dereference (value);
+		    if (aborting)
+			    break;
 		    v = ValueIncDec (v, inst->binop.op);
 		    ThreadAssign (value, v, False);
 		    value = v;
 		    break;
 		case OpPostOp:
 		    v = Dereference (value);
+		    if (aborting)
+			    break;
 		    ThreadAssign (value, ValueIncDec (v, inst->binop.op), False);
 		    value = v;
 		    break;
