@@ -29,6 +29,10 @@
 #include	<sys/resource.h>
 #endif
 
+#if HAVE_LIBREADLINE
+#include <readline/readline.h>
+#endif
+
 int	stdin_interactive;
 
 static void
@@ -161,6 +165,9 @@ intr (int sig)
     if (signalInterrupt) {
 	int ret = write(2,"Double interrupt, exiting\n", 26);
 	(void) ret;
+#if HAVE_RL_CLEANUP_AFTER_SIGNAL
+	rl_cleanup_after_signal();
+#endif
 	exit(1);
     }
     SetSignalInterrupt ();
@@ -171,6 +178,9 @@ stop (int sig)
 {
     sigset_t	set, oset;
 
+#if HAVE_RL_CLEANUP_AFTER_SIGNAL
+    rl_cleanup_after_signal();
+#endif
     sigfillset (&set);
     sigprocmask (SIG_SETMASK, &set, &oset);
     IoStop ();
@@ -182,12 +192,18 @@ stop (int sig)
     sigprocmask (SIG_SETMASK, &oset, &set);
     IoStart ();
     catchSignal (sig, stop);
+#if HAVE_RL_RESET_AFTER_SIGNAL
+    rl_reset_after_signal();
+#endif
 }
 
 void
 die (int sig)
 {
     IoStop ();
+#if HAVE_RL_CLEANUP_AFTER_SIGNAL
+    rl_cleanup_after_signal();
+#endif
     _exit (sig);
 }
 
@@ -195,6 +211,9 @@ void
 segv (int sig)
 {
     IoStop ();
+#if HAVE_RL_CLEANUP_AFTER_SIGNAL
+    rl_cleanup_after_signal();
+#endif
     releaseSignal (SIGSEGV);
     /* return and reexecute the fatal instruction */
 }
